import '../lib/polyfill';

import React from 'react';
import { render } from 'react-dom';
import { injectGlobal } from 'styled-components';
import MediaEdit from '../components/media-edit';
import BannerEdit from '../components/banner-edit';
import PerfectScrollbar from 'perfect-scrollbar';
import 'perfect-scrollbar/css/perfect-scrollbar.css';
import DispatchSystemUpdate from '../admin/system-update';
import dispatchMediaInsert from '../admin/media-insert';
import dispatchMediaField from '../admin/media-insert-field';
import DispatchStaticExport from '../admin/static-export';
import dispatchEntryEditor from '../admin/entry-editor';

/* eslint-disable no-unused-expressions */
injectGlobal` 
  body .Select-multi-value-wrapper>.Select-value {
    background-color: #EDEDED;
    color: #5A5A5A;
    border: 1px solid #B7B7B7;
    line-height: 1.8;
    margin-top: 3px;
    .Select-value-icon {
      border-right: 1px solid #B7B7B7;
      &:hover {
        background-color: inherit;
        color: inherit;
      }
    }
  }
`;
/* eslint-disable no-unused-expressions */

ACMS.Ready(() => {
  __webpack_public_path__ = ACMS.Config.root; // eslint-disable-line

  /**
   * Delayed contents event
   */
  ACMS.addListener('acmsAdminSelectTab', (e) => {
    ACMS.dispatchEvent('acmsAdminDelayedContents', e.target, e.obj);
  });
  ACMS.addListener('acmsAdminShowTabPanel', (e) => {
    ACMS.dispatchEvent('acmsAdminDelayedContents', e.target, e.obj);
  });
  ACMS.addListener('acmsDialogOpened', (e) => {
    ACMS.dispatchEvent('acmsAdminDelayedContents', e.target, e.obj);
  });

  /**
   * モーダルないのいらない情報を削除
   */
  ACMS.addListener('acmsDialogOpened', (e) => {
    $('.js-hide-on-modal', e.target).remove();
  });

  /**
   * エントリー編集画面
   */
  dispatchEntryEditor();

  /**
   * カスタムフィールドメーカー
   */
  const fieldMaker = document.querySelector('#custom-field-maker');
  if (fieldMaker) {
    import(/* webpackChunkName: "custom-field-maker-css" */'custom-field-maker/src/css/custom-field-maker.css');
    import(/* webpackChunkName: "custom-field-maker" */'custom-field-maker').then(({ default: CustomFieldMaker }) => {
      render(<CustomFieldMaker />, fieldMaker);
    });
  }

  /**
   * OpenSereetMap
   */
  const dispatchGeoPicker = (item) => {
    import(/* webpackChunkName: "geo-picker" */'../lib/geo-picker').then(({ default: geoPicker }) => {
      geoPicker(item);
    });
  };
  const openStreetMapsPickers = document.querySelectorAll('.js-open-street-map-editable');
  [].forEach.call(openStreetMapsPickers, (item) => {
    dispatchGeoPicker(item);
  });
  ACMS.addListener('acmsAddCustomFieldGroup', (e) => {
    dispatchGeoPicker(e.obj.item);
  });
  ACMS.addListener('acmsAddUnit', (e) => {
    dispatchGeoPicker(e.obj.item);
  });
  ACMS.addListener('onGeoInfoAdded', (e) => {
    dispatchGeoPicker(e.target);
  });

  /**
   * メニューのスクロールバー（IE以外）
   */
  if (!/^ie/.test(ACMS.Dispatch.Utility.getBrowser())) {
    const psDom = document.querySelector('.js-scroll-contents');
    if (psDom) {
      const ps = new PerfectScrollbar(psDom, {
        wheelSpeed: 1,
        wheelPropagation: true,
        minScrollbarLength: 20
      });
      ps.update();
    }
  }

  /**
   * Color picker（IE10以上）
   */
  const dispatchColorPicer = () => {
    if (ACMS.Dispatch.Utility.browser().ltIE9) {
      const buttons = document.querySelectorAll('.js-acms-color-picker-submit');
      [].forEach.call(buttons, (button) => {
        button.parentNode.removeChild(button);
      });
    } else {
      const colorPickers = document.querySelector('.js-acms-color-picker');
      if (colorPickers) {
        import(/* webpackChunkName: "color-picker" */'../admin/color-picker').then(({ default: DispatchColorPicker }) => {
          DispatchColorPicker();
        });
      }
    }
  };
  ACMS.addListener('acmsAdminDelayedContents', () => {
    dispatchColorPicer();
  });
  dispatchColorPicer();


  /**
   * クイックサーチ（IE10以上）
   */
  const dispatchQuickSearch = () => {
    import(/* webpackChunkName: "quick-search" */'../components/quick-search').then(({ default: QuickSearch }) => {
      if (ACMS.Dispatch.Utility.browser().ltIE9 || ACMS.Config.auth === 'subscriber') {
        const buttons = document.querySelectorAll('.js-search-everything');
        [].forEach.call(buttons, (button) => {
          button.parentNode.removeChild(button);
        });
      } else if (ACMS.Config.quickSearchFeature === true && !/(\?|&)acms-preview-mode=/.test(location.href)) {
        document.body.insertAdjacentHTML('beforeend', "<div id='acms-search-everything'></div>");
        const quickSearch = document.querySelector('#acms-search-everything');
        render(<QuickSearch />, quickSearch);
      }
    });
  };
  const previewTarget = document.querySelector('.js-acms-preview');
  if (!previewTarget) {
    dispatchQuickSearch();
  }

  /**
   * プレビュー, タイムマシンモード
   */
  import(/* webpackChunkName: "preview" */'../lib/preview').then(({ default: preview }) => {
    preview();
  });

  /**
   * 擬似フォーム
   */
  const dispatchFakeForms = () => {
    const fakeForms = document.querySelectorAll('.js-fake-form');
    [].forEach.call(fakeForms, (item) => {
      const submit = item.querySelector('.js-submit');
      $(submit).unbind('click').bind('click', (e) => {
        e.preventDefault();

        const method = item.getAttribute('data-method');
        const form = document.createElement('form');

        form.setAttribute('method', method);
        form.style.display = 'none';

        $(item).find(':input, :radio, :checkbox, :submit').each((i, elm) => {
          const copy = elm.cloneNode(true);
          copy.value = elm.value;
          form.appendChild(copy);
        });
        document.body.appendChild(form);
        form.submit();
      });
    });
    $('form').submit(function () {
      $(this).find('.js-fake-form').remove();
    });
  };
  ACMS.addListener('acmsAdminDelayedContents', () => {
    dispatchFakeForms();
  });
  dispatchFakeForms();


  /**
   * System update
   */
  DispatchSystemUpdate();

  /**
   * Static export
   */
  DispatchStaticExport();


  /**
   * 固定ナビゲーション
   */
  const scrollAreaHeight = $('.js-scroll-fixed').height();
  $('.js-scroll-contents').css('padding-bottom', scrollAreaHeight);

  /**
   * Password strength checker
   */
  const passwordStrength = document.querySelectorAll(ACMS.Config.passwordStrengthMark);
  if (passwordStrength.length > 0) {
    import(/* webpackChunkName: "zxcvbn" */'../lib/zxcvbn').then(({ default: zxcvbn }) => {
      [].forEach.call(passwordStrength, (item) => {
        zxcvbn(item);
      });
    });
  }

  /**
   * Select2
   */
  const dispatchSelect2 = (elm) => {
    const target = elm || document;
    const selectBox = target.querySelectorAll(ACMS.Config.select2Mark);
    [].forEach.call(selectBox, (item) => {
      const option = {
        containerCssClass: 'acms-admin-selectbox',
        dropdownCssClass: 'acms-admin-select-dropdown',
      };
      if (item.querySelectorAll('option').length >= ACMS.Config.select2Threshold) {
        import(/* webpackChunkName: "select2" */'../lib/select2').then(({ default: select2 }) => {
          select2(item, option);
        });
      }
    });
  };
  dispatchSelect2(document);
  ACMS.addListener('acmsAdminDelayedContents', (e) => {
    dispatchSelect2(e.target);
  });
  ACMS.addListener('acmsAddUnit', (e) => {
    dispatchSelect2(e.obj.item);
  });
  ACMS.addListener('acmsAddCustomFieldGroup', (e) => {
    dispatchSelect2(e.obj.item);
  });

  /**
   * Disclose password
   */
  const disclosePassword = document.querySelectorAll('.js-disclose_password');
  [].forEach.call(disclosePassword, (item) => {
    item.addEventListener('change', (event) => {
      const selector = event.target.getAttribute('data-target');
      const target = document.querySelector(selector);
      if (event.target.checked) {
        target.setAttribute('type', 'text');
      } else {
        target.setAttribute('type', 'password');
      }
    });
  });

  /**
   * AdminMenu editor
   */
  const dispatchAdminMenuEditor = (elm) => {
    const adminMenuEditor = elm.querySelector(ACMS.Config.adminMenuEditMark);
    if (adminMenuEditor) {
      import(/* webpackChunkName: "admin-menu" */'../components/admin-menu').then(({ default: AdminMenuEdit }) => {
        const json = JSON.parse(document.querySelector('#admin-menu-json').innerHTML);
        json.cards = json.cards.filter((card => card.id));
        json.lanes = json.lanes.filter((lane => lane.id));
        const { lanes: laneArry } = json;
        const lanes = laneArry.map((lane) => {
          const cards = json.cards.filter(card => card.laneId === lane.id);
          return { ...lane, cards };
        });
        if (lanes[0]) {
          lanes[0].draggable = false;
        }
        render(<AdminMenuEdit data={{ lanes }} />, adminMenuEditor);
      });
    }
  };
  dispatchAdminMenuEditor(document);
  ACMS.addListener('acmsAdminDelayedContents', () => {
    dispatchAdminMenuEditor(document);
  });

  /**
   * Navigation editor
   */
  const dispatchNavigationEditor = (elm) => {
    const navigationEditor = elm.querySelector(ACMS.Config.navigationEditMark);
    if (navigationEditor && !/^ie/.test(ACMS.Dispatch.Utility.getBrowser())) {
      import(/* webpackChunkName: "navigation-edit" */'../components/navigation-edit').then(({ default: NavigationEdit }) => {
        $('.js-navigation-ie').remove();
        const json = document.querySelector('#js-navigation-json');
        const items = JSON.parse(json.innerHTML);

        import(/* webpackChunkName: "html-entities" */'html-entities').then(({ AllHtmlEntities: Entities }) => {
          const entities = new Entities();
          items.forEach((item) => {
            item.navigation_attr = entities.decode(item.navigation_attr);
            item.navigation_a_attr = entities.decode(item.navigation_a_attr);
            item.navigation_label = entities.decode(item.navigation_label);
          });
          render(
            <NavigationEdit
              items={items}
              message={ACMS.Config.navigationMessage
              }
            />,
            navigationEditor
          );
        });
      });
    } else {
      $('.js-navigation-ie').show();
    }
  };
  dispatchNavigationEditor(document);
  ACMS.addListener('acmsAdminDelayedContents', () => {
    dispatchNavigationEditor(document);
  });

  const dispatchMediaEditor = (elm) => {
    const mediaEditor = elm.querySelector(ACMS.Config.mediaAdminMark);
    if (mediaEditor) {
      render(<MediaEdit />, mediaEditor);
    }
  };
  dispatchMediaEditor(document);
  ACMS.addListener('acmsAdminDelayedContents', () => {
    dispatchMediaEditor(document);
  });
  /**
   * Banner Editor
   */
  const dispatchBannerEditor = (elm) => {
    const bannerEditor = elm.querySelector(ACMS.Config.bannerEditMark);
    if (!bannerEditor) {
      return;
    }
    import(/* webpackChunkName: "html-entities" */'html-entities').then(({ AllHtmlEntities: Entities }) => {
      $('.js-banner-ie').remove();
      const ele = document.querySelector('#js-banner-json');
      const json = ele.innerHTML;
      const decoded = Entities.decode(json);
      const replaced = decoded.replace(/\\\//g, '/');
      const items = JSON.parse(replaced);
      const entities = new Entities();
      const attr1 = $('.js-label-attr1').val();
      const attr2 = $('.js-label-attr2').val();
      items.forEach((item) => {
        item.banner_attr = entities.decode(item.banner_attr);
        item.banner_a_attr = entities.decode(item.banner_a_attr);
        item.banner_label = entities.decode(item.banner_label);
      });
      render(
        <BannerEdit
          attr1={attr1}
          attr2={attr2}
          items={items}
          message={ACMS.Config.bannerMessage}
        />,
        bannerEditor
      );
    });
  };
  dispatchBannerEditor(document);
  ACMS.addListener('acmsAdminDelayedContents', () => {
    dispatchBannerEditor(document);
  });

  dispatchMediaField(document);
  ACMS.addListener('acmsAddUnit', (event) => {
    dispatchMediaInsert(event.target);
    dispatchMediaField(event.target);
  });
  ACMS.addListener('acmsAddCustomFieldGroup', (event) => {
    dispatchMediaField(event.target);
  });
  ACMS.addListener('acmsAdminDelayedContents', (event) => {
    dispatchMediaField(event.target);
  });
  ACMS.addListener('acmsCustomFieldMakerPreview', (event) => {
    dispatchMediaField(event.target);
  });

  /**
   * Dispatch acmsAdminReady
   */
  ACMS.dispatchEvent('acmsAdminReady');

  /**
   * GeoPicker
   */
  const dispatchGeoInput = () => {
    const $geoInput = $('.js-geo_input');
    const $geoForm = $('.js-geo_form');
    if ($geoInput.size() && $geoForm.size()) {
      const $lat = $geoForm.find('[name=geo_lat]');
      const $lng = $geoForm.find('[name=geo_lng]');
      const $zoom = $geoForm.find('[name=geo_zoom]');
      const defaultLat = ACMS.Config.adminLocationDefaultLat ? ACMS.Config.adminLocationDefaultLat : '';
      const defaultLng = ACMS.Config.adminLocationDefaultLng ? ACMS.Config.adminLocationDefaultLng : '';
      const defaultZoom = ACMS.Config.adminLocationDefaultZoom ? ACMS.Config.adminLocationDefaultZoom : '10';
      const addGeo = function () {
        $geoForm.show();
        $geoInput.val(ACMS.i18n('geo.message1'));
        $geoInput.data('type', 'add');
        if (!$lat.val()) {
          $lat.val(defaultLat);
        }
        if (!$lng.val()) {
          $lng.val(defaultLng);
        }
        if (!$zoom.val()) {
          $zoom.val(defaultZoom);
        }
        ACMS.dispatchEvent('onGeoInfoAdded', $geoForm.get(0));
      };
      const deleteGeo = function () {
        $geoForm.hide();
        $geoInput.val(ACMS.i18n('geo.message2'));
        $geoInput.data('type', 'delete');
        $lat.val('');
        $lng.val('');
        $zoom.val('');
      };

      $geoInput.bind('click', () => {
        const type = $geoInput.data('type');
        if (type === 'add') {
          deleteGeo();
        } else if (type === 'delete') {
          addGeo();
        }
      }).trigger('click');
    }
  };
  dispatchGeoInput();

  /**
   * スタイルガイド
   */
  if ($('.js-navSubOpener').length > 0) {
    $('.js-navSubOpener').click(() => {
      $('.navSubGroup').toggleClass('acms-admin-block');
      return false;
    });

    const calcHeight = () => {
      const adminNav = $('.acms-navbar-admin').height() + 40;
      const height = $(window).height() - adminNav;
      $('.navSubGroup').height(`${height}px`);
    };

    calcHeight();

    $(window).resize(calcHeight);

    $('.js-sample-start').click(function () {
      $(this).parent().parent().toggleClass('active');
    });
  }
});

